#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from cobbler import api
from xmlrpclib import ServerProxy
import sys

class RevisorCobbler(api.BootAPI):
    """ Revisor class to interact with cobbler. """

    def newProfile(self, name, distro, kickstart):
        """ A Simple call to interact with the base cobbler API to add a profile. """
        # cobbler profile add --name=string --distro=string [--kickstart=url] [--kopts=string] [--ksmeta=string] [--virt-file-size=gigabytes]
        # [--virt-ram=megabytes]
        profile_info = {'name': name, 'distro': distro, 'kickstart': kickstart}
        profile = self.new_profile()
        profile.from_datastruct(profile_info)
        self.profiles().add(profile, with_copy=self.sync_flag)
        self.saveChanges()

    def addCompose(self, location, name, callback):
        """ Add the compose to cobbler. This does both newDistro() and newProfile() """
        ostdout = sys.stdout
        sys.stdout = callback
        self.import_tree(location, name)
        sys.stdout = ostdout

    def listOptions(self, server=None, port=None):
        """ A Simple call to interact with the base cobbler API to return a list of what distros and profiles we have access to. """
        # cobbler list
        retList = []
        if not server:
            for d in self.distros():
                distro = {"name": d.name}
                profiles = []
                for p in self.profiles():
                    if p.distro == d.name:
                        profiles.append(p.name)
                distro["profiles"] = profiles
                retList.append(distro)
        else:
            #FIXME: Add try: except: to catch errors with communicating with server, give a nice error mesage.
            if not port:
                port = 25151
            server_url = "http://%s:%s" % (server, port)
            cobblerserver = RevisorCobblerServer(server_url)
            for d in cobblerserver.get_distros():
                distro = {"name": d["name"]}
                profiles = []
                for p in cobblerserver.get_profiles():
                    if p["distro"] == d["name"]:
                        profiles.append(p["name"])
                distro["profiles"] = profiles
                retList.append(distro)

        return retList

    def saveChanges(self):
        """ Do what needs to be done to save all our changes. """
        self.serialize()
        self.sync()

class RevisorCobblerServer(ServerProxy):

    def __init__(self, url=None):
        ServerProxy.__init__(self, url, allow_none=True)

